==> saphire-0.9.1.tgz: saphire-0.9.1/saphire_inner.h <==
#if !defined(SAPHIRE_INNER_H)
#define SAPHIRE_INNER_H

////////////////////////////////////////////////////////////
/// from saphire_main.c
////////////////////////////////////////////////////////////
extern volatile BOOL gSigUser;

extern hash_obj* gMatchRestart;
    // matchǻȤѿ

/// ޥɤбʸ
enum eCommandKind { 
    kSubshell, kMSleep, kTrue, kFalse, kTest,
    kIndex, kRIndex, kLength, kLc, kUc,
    kChomp, kSubstr, kEval, kFg, kBg, kCpg,
    kJobs, kRehash, kKanjiCode, kLineField, kVar, kGlobal,
    kCalc, kExport, kPrint,  kPuts, kLoad, 
    kWhile, kBreak, kExit, kIf, kSplit, kX, kJoin, kLines, kRows,
    kAryNew, kAryAdd, kAryErase, kAryClear, 
    kHashNew, kHashAdd, kHashErase, kHashClear, kMatch, kPos, kSub, kScan,
    kRead, 
    kCd, 
    kAdd, kDel, kSelector, kMax, kMin,
    kExtName, kParentName, kNoExtName, 
    kCompile, kRaise,
    kPopd, kPushd,
    kPlusPlus,
    kMinusMinus,
    kPlus,
    kMinus,
    kMult,
    kDiv,
    kMod,
    kPow,
    kSort,
    kRange,
    kPrintf,
    kPomch,
    kDef,
    kPTee,
    kPCat,
    kCommand 
};

string_obj* gAncestalMemoryPipeIn;
    // ȥåץ٥ޥѤΥѥ
string_obj* gAncestalMemoryPipeOut;
    // ȥåץ٥ޥѤΥѥ

char gTmpDir[PATH_MAX];
    // ե֤

extern char* gStatmentKindStrs[kCommand+1];
extern hash_obj* gFuncs;
    // 桼ؿ sFunction*äƤ
    // ϴؿ̾
extern hash_obj* gGlobals;
    // Хѿ ѿ̾ Ƥstring_obj*
extern hash_obj* gArrays;
    // Хѿ ѿ̾
    // Ƥ(vector_obj*) ˤstring_obj*
    // äƤ
hash_obj* gHashs;
    // Хѿ ѿ̾
    // Ƥϥϥå(hash_obj*) ϥåˤstring_obj*
    // äƤ

extern vector_obj* gStackFrame;
    // ѿΥå
    // Ƥhash_obj*
    // Υϥѿ̾
    // Ƥstring_obj*
extern enum eAppType gAppType;
    // ץꥱμ
extern BOOL gJobControl;
    // ֤Υȥ򤹤뤫ɤ
extern string_obj* gFName;
    // ץȤΥե̾
extern int gLine;
    // ץȤιֹ
extern vector_obj* gJobs;
    //  sJob*
extern list_obj* gSigChldRecieved;  
    // SIGCHLDäPID,STATUSΥꥹ

extern char* gMagicNumber;
    // ֥ȥեΥޥåʥС

extern hash_obj* gFileHandles;
    //  ե̾
    //  եǥץ
    // ֥ȥեΥޥåʥС

extern int gGPipeStat; // 0:̵ 1: ꡼
    // Хѥפξ

string_obj* gGlobalPipe;
    // Хѥפɤ߹ߤǡ
    // Хѥ

extern hash_obj* gRegexs;
    // ɽΥå
    // ɽʸ
    // Ƥregex_t

extern hash_obj* gRegexsI;
    // ɽΥå(ignore case)
    // ɽʸ
    // Ƥregex_t

extern hash_obj* gRegexsM;
    // ɽΥå(multi line)
    // ɽʸ
    // Ƥregex_t

extern hash_obj* gRegexsIM;
    // ɽΥå(ignore case|multi line)
    // ɽʸ
    // Ƥregex_t

extern hash_obj* gFReadBuffers;
    // ɤ߹ߥХåե
    // ϥեǥץֹʸ
    // Ƥɤ߹ե
    // statment_tree_internal_commands_read_nextin_oneline
    // ǻȤ
extern hash_obj* gFWriteBuffers;
    // 񤭹ߥХåե
    // ϥեǥץֹʸ
    // ƤϽ񤭹ե

extern int gMsgId;
    // å塼
    // fork˿ƥץȻҥץ̿˻Ȥ
    // (ץ롼פѤä)

extern hash_obj* gInnerCommands;
    // Ͽޥ
    // saphireȤ߹ץꥱ
    // 줿ޥɤΥǡ
    // ϥޥ̾
    // ƤsInnerCommand

extern vector_obj* gDirStack;
    // popd, pushdǻȤǥ쥯ȥꥹå
    // Ƥ string_obj*
extern enum eLineField gLineField;
    // saphireʸޥɤǻȤԥ

extern vector_obj* gPSubTmpFiles;
    // ץִǻȤեǾä
    // ¸Ƥ

BOOL is_line_field(char* p);
    // 饤եɤݥȤƤ뤫

BOOL is_line_field2(enum eLineField lf, char* p);
    // 饤եɤݥȤƤ뤫2

BOOL string_chomp(string_obj* str);
    // ԥɤ

void string_ponch(string_obj* str);
    // ԥɤդ(\n)

void string_pomch(string_obj* str);
    // ԥɤդ(ԥɤΤޤ)

BOOL xclose(int fd);
    // closeȤˤΥեǥץ
    // gFReadBuffersΥХåեк
    // gFWriteBuffersΥХåեн񤭹Ǻ
    // gFileHandles˥եϥɥ뤬к

void update_ary_env(char* key);
    // ѿˤޤĤѿ(_size)ͤ˹

void clear_ary_env(char* key);
    // ѿˤޤĤѿä

void saphire_delete_same_name_var(char* name);
    // Ʊ̾ΥХϤѿƾä

void err_msg(char* msg);
    // 顼å

int get_command_kind(char* arg0);
    // ޥ̾饳ޥIDõ

void null_fun();
    // gotoΥ٥򥳥ѥ顼̤δؿ

BOOL parse_is_redirect(char* p);
    // ݥ󥿡ʸ󤬥쥯Ȥɤ

BOOL parse_is_redirect2(char* p);
    // ݥ󥿤ʸ󤬥쥯Ȥɤ2

BOOL is_env_name_char(char c);
    // ѿǻȤʸɤ

BOOL is_glob_terminated_char(char* p);
    // ֤ζڤʸɤ

BOOL is_terminated_char(char* p);
    // 

void get_quoted_linefield(char* str, string_obj* out);

typedef struct _sJob {
    string_obj* mName;          // ֤̾
    pid_t mPGroup;              // ץ롼ID
    vector_obj* mPIDs;          // ҥץPIDΥꥹ
    struct termios* mTty;
} sJob;

sJob* sJob_new();
void sJob_delete(sJob* self);

enum eStatmentTerminated { kTNormal, kTOrOr, kTAndAnd };

typedef struct
{
    sStatments* joukensiki;
    sStatments* contents;
} sWhile;

sWhile* sWhile_new(MANAGED sStatments* joukensiki, MANAGED sStatments* contents);

void sWhile_delete(sWhile* self);
void sWhile_save(sWhile* self, int fd);
sWhile* sWhile_load(int fd);

typedef struct
{
    sStatments* contents;
} sSubshell;

sSubshell* sSubshell_new(MANAGED sStatments* contents);

void sSubshell_delete(sSubshell* self);
void sSubshell_save(sSubshell* self, int fd);
sSubshell* sSubshell_load(int fd);
void sSubshell_view(sSubshell* self);

typedef struct
{
    vector_obj* joukensiki_list;
    vector_obj* contents_list;
} sIf;

sIf* sIf_new(MANAGED vector_obj* joukensiki_list, MANAGED vector_obj* contents_list);

void sIf_delete(sIf* self);
void sIf_save(sIf* self, int fd);
sIf* sIf_load(int fd);

enum eCommandKind2 {
    kCKLocalVar, kCKUserCommand, kCKExternInnerCommand, kCKInnerCommand,
    kCKExternCommand
};

enum eRedirect { kRedirectInput, kRedirectOverwrite, kRedirectAppend
          , kRedirectErrAndOutput };

typedef struct {
    enum eRedirect mType;
    int mFd;
    string_obj* mFName;
} sRedirect; // ¹Ի˻Ȥ ֥ȥե¸

sRedirect* sRedirect_new(enum eRedirect type, int fd, char* fname);
void sRedirect_save(sRedirect* self, int fd);
sRedirect* sRedirect_load(int fd);
void sRedirect_delete(sRedirect* self);

typedef struct {
    enum eRedirect mType;
    int mFd;
} sRedirectTmp; // ѡǰŪ˻Ȥ

sRedirectTmp* sRedirectTmp_new(enum eRedirect type, int fd);
void sRedirectTmp_delete(sRedirectTmp* self);

typedef struct {
    string_obj* mName;

    string_obj* mIndex;
    string_obj* mIndex2;
    string_obj* mDelimiter;
} sVar;

sVar* sVar_new(MANAGED string_obj* name,  MANAGED string_obj* index, MANAGED string_obj* index2, MANAGED string_obj* delimiter);
void sVar_delete(sVar* self);

sVar* sVar_load(int fd);
void sVar_save(sVar* self, int fd);

typedef struct
{
    int mKind;   // 0: string 1: var 2: redirect 3: statments
    void* mBody;
} sArgTmp; // ѡǰŪ˻Ȥ

sArgTmp* sArgTmp_new(int kind, void* body);
void sArgTmp_delete(sArgTmp* arg);

typedef struct
{
    int mKind; // 0:string 1: var 2: statments
    void* mBody;
} sArg; // mArgs

sArg* sArg_new(int kind, void* body);
void sArg_delete(sArg* arg);

void sArg_save(sArg* self, int fd);
sArg* sArg_load(int fd);

typedef struct
{
    enum eCommandKind mKind;
    vector_obj* mArgs;
        // vector of sArg*
        // ޥɥ饤
    vector_obj* mRedirects;
        // vector of sRedirect* 
        // 쥯Υꥹ

    void* mExtra;

    vector_obj* mArgTmps;
        // vector of sArgTmp*
        // ѡѤ˰Ū˻Ȥѿ
} sCommand;

#ifdef MDEBUG 
sCommand* sCommand_new(const char* file, int line, const char* function);
#define COMMAND_NEW() sCommand_new(__FILE__, __LINE__, __FUNCTION__)
void sCommand_view(sCommand* self);
#else
sCommand* sCommand_new();
#define COMMAND_NEW() sCommand_new()
#endif

void sCommand_save(sCommand* self, int fd);
sCommand* sCommand_load(int fd);
void sCommand_delete(sCommand* self);

typedef struct
{
    vector_obj* mCommands;   // ޥ

    string_obj* mFName;      // ʸե̾
    int mLine;               // ʸ

    string_obj* mNotEvaled;  // run˥ѡʸ
                             // Ķѿȥޥޤʸ
    string_obj* mTitle;      // ȥ

    enum eStatmentTerminated mTerminated;   
            // ʣʸκǸ夬ǶڤƤ뤫

    BOOL mRCodeReverse;      // ꥿󥳡ɤȿž

    BOOL mBackground;        // Хå饦ɤǼ¹

    BOOL mGlobalPipeIn;   // ХѥפϤ뤫
    BOOL mGlobalPipeOut;  // ХѥפνϤ뤫
} sStatment;

#ifdef MDEBUG

sStatment* sStatment_new(const char* file, int line, const char* function);
#define STATMENT_NEW() sStatment_new(__FILE__, __LINE__, __FUNCTION__)
void sStatment_view(sStatment* self);

#else

sStatment* sStatment_new();
#define STATMENT_NEW() sStatment_new()

#endif

void sStatment_delete(sStatment* self);
void sStatment_save(sStatment* self, int fd);
sStatment* sStatment_load(int fd);

#ifdef MDEBUG
sStatments* sStatments_new(const char* file, int line, const char* function);

#define STATMENTS_NEW() sStatments_new(__FILE__, __LINE__, __FUNCTION__)
#else

sStatments* sStatments_new();

#define STATMENTS_NEW() sStatments_new()

#endif

void sStatments_view(sStatments* self);

void sStatments_save(sStatments* self, int fd);
sStatments* sStatments_load(int fd);
void sStatments_delete(sStatments* self);

typedef struct {
    string_obj* name;       // ޥ̾
    string_obj* arg_name;
    sStatments* statments; // ̿ɽ
} sFunction;

sFunction* sFunction_new(char* name, MANAGED sStatments* statments, char* arg_name);
  
void sFunction_delete(sFunction* self);
void sFunction_save(sFunction* self, int fd);
sFunction* sFunction_load(int fd);
void sFunction_view(sFunction* self);

typedef struct
{
    int mPid;
    int mStatus;
    int mPGroup;
} sOtherCProg;

extern vector_obj* gOtherCProgs;

sOtherCProg* sOtherCprog_new(int pid, int status, int pgroup);
void sOtherCProg_delete(sOtherCProg* self);

typedef struct {
    pid_t mPid;
    string_obj* mTitle;
} sChildProgram;

sChildProgram* sChildProgram_new(pid_t pid, char* title);
void sChildProgram_delete(sChildProgram* self);

typedef struct {
    string_obj* mName;
    fInnerCommand mFun;
} sInnerCommand;

sInnerCommand* sInnerCommand_new(char* name, fInnerCommand fun);
void sInnerCommand_delete(sInnerCommand* self);

void sigchld_block(int block);

////////////////////////////////////////////////////////////
/// from saphire_commands.c
////////////////////////////////////////////////////////////
BOOL statment_tree_internal_commands(sCommand* command, int* rcode, vector_obj* argv, sWFd* nextout, sRFd* nextin, int nexterr, int j, vector_obj* cprogs, char* title, BOOL input);
BOOL vcat_expand_env(string_obj* pipeout, char* env, vector_obj* numbers);

////////////////////////////////////////////////////////////
/// from saphire_parser.c
////////////////////////////////////////////////////////////
BOOL expand_env(char* p, string_obj* cmdline2);
int parse(char* cmdline, char* fname, sStatments* statments);
BOOL read_expand_brace_expansion(char* p, string_obj* cmdline2, char* separator);
BOOL parse_number(char* p, vector_obj* numbers);

typedef struct {
    int number;
    int number2;
    int number_num;
} sIndex;

sIndex* sIndex_new(int num, int num2, int num_num);
void sIndex_delete(sIndex* self);

BOOL read_until_next_cbrace_end(char** p, string_obj* not_evaled, int* read_end_of_statment);

////////////////////////////////////////////////////////////
// from saphire_vm.c
////////////////////////////////////////////////////////////
BOOL forground_job(int num);
void background_job(int num);
void tmpfile_all_sweep();

#endif

==> saphire-0.9.2.tgz: saphire-0.9.2/saphire_inner.h <==
#if !defined(SAPHIRE_INNER_H)
#define SAPHIRE_INNER_H

////////////////////////////////////////////////////////////
/// from saphire_main.c
////////////////////////////////////////////////////////////
extern volatile BOOL gSigUser;

extern hash_obj* gMatchRestart;
    // matchǻȤѿ

/// ޥɤбʸ
enum eCommandKind { 
    kSubshell, kMSleep, kTrue, kFalse, kTest,
    kIndex, kRIndex, kLength, kLc, kUc,
    kChomp, kSubstr, kEval, kFg, kBg, kCpg,
    kJobs, kRehash, kKanjiCode, kLineField, kVar, kGlobal,
    kCalc, kExport, kPrint,  kPuts, kLoad, 
    kWhile, kBreak, kExit, kIf, kSplit, kX, kJoin, kLines, kRows,
    kAryNew, kAryAdd, kAryErase, kAryClear, 
    kHashNew, kHashAdd, kHashErase, kHashClear, kMatch, kScan, kErase, kPos, kSub,
    kRead, 
    kCd, 
    kAdd, kDel, kSelector, kMax, kMin,
    kExtName, kParentName, kNoExtName, 
    kCompile, kRaise,
    kPopd, kPushd,
    kPlusPlus,
    kMinusMinus,
    kPlus,
    kMinus,
    kMult,
    kDiv,
    kMod,
    kPow,
    kSort,
    kRange,
    kPrintf,
    kPomch,
    kDef,
    kPTee,
    kPCat,
    kCommand 
};

void err_msg(char* msg);
    // 顼å

string_obj* gAncestalMemoryPipeIn;
    // ȥåץ٥ޥѤΥѥ
string_obj* gAncestalMemoryPipeOut;
    // ȥåץ٥ޥѤΥѥ

char gTmpDir[PATH_MAX];
    // ե֤

extern char* gStatmentKindStrs[kCommand+1];
extern hash_obj* gFuncs;
    // 桼ؿ sFunction*äƤ
    // ϴؿ̾
extern hash_obj* gGlobals;
    // Хѿ ѿ̾ Ƥstring_obj*
extern hash_obj* gArrays;
    // Хѿ ѿ̾
    // Ƥ(vector_obj*) ˤstring_obj*
    // äƤ
hash_obj* gHashs;
    // Хѿ ѿ̾
    // Ƥϥϥå(hash_obj*) ϥåˤstring_obj*
    // äƤ

extern vector_obj* gStackFrame;
    // ѿΥå
    // Ƥhash_obj*
    // Υϥѿ̾
    // Ƥstring_obj*
extern enum eAppType gAppType;
    // ץꥱμ
extern BOOL gJobControl;
    // ֤Υȥ򤹤뤫ɤ
extern string_obj* gFName;
    // ץȤΥե̾
extern int gLine;
    // ץȤιֹ
extern vector_obj* gJobs;
    //  sJob*
extern list_obj* gSigChldRecieved;  
    // SIGCHLDäPID,STATUSΥꥹ

extern char* gMagicNumber;
    // ֥ȥեΥޥåʥС

extern hash_obj* gFileHandles;
    //  ե̾
    //  եǥץ
    // ֥ȥեΥޥåʥС

extern int gGPipeStat; // 0:̵ 1: ꡼
    // Хѥפξ

string_obj* gGlobalPipe;
    // Хѥפɤ߹ߤǡ
    // Хѥ

extern hash_obj* gRegexs;
    // ɽΥå
    // ɽʸ
    // Ƥregex_t

extern hash_obj* gRegexsI;
    // ɽΥå(ignore case)
    // ɽʸ
    // Ƥregex_t

extern hash_obj* gRegexsM;
    // ɽΥå(multi line)
    // ɽʸ
    // Ƥregex_t

extern hash_obj* gRegexsIM;
    // ɽΥå(ignore case|multi line)
    // ɽʸ
    // Ƥregex_t

extern hash_obj* gFReadBuffers;
    // ɤ߹ߥХåե
    // ϥեǥץֹʸ
    // Ƥɤ߹ե
    // statment_tree_internal_commands_read_nextin_oneline
    // ǻȤ
extern hash_obj* gFWriteBuffers;
    // 񤭹ߥХåե
    // ϥեǥץֹʸ
    // ƤϽ񤭹ե

extern int gMsgId;
    // å塼
    // fork˿ƥץȻҥץ̿˻Ȥ
    // (ץ롼פѤä)

extern hash_obj* gInnerCommands;
    // Ͽޥ
    // saphireȤ߹ץꥱ
    // 줿ޥɤΥǡ
    // ϥޥ̾
    // ƤsInnerCommand

extern vector_obj* gDirStack;
    // popd, pushdǻȤǥ쥯ȥꥹå
    // Ƥ string_obj*
extern enum eLineField gLineField;
    // saphireʸޥɤǻȤԥ

extern vector_obj* gPSubTmpFiles;
    // ץִǻȤեǾä
    // ¸Ƥ

BOOL is_line_field(char* p);
    // 饤եɤݥȤƤ뤫

BOOL is_line_field2(enum eLineField lf, char* p);
    // 饤եɤݥȤƤ뤫2

BOOL string_chomp(string_obj* str);
    // ԥɤ

void string_ponch(string_obj* str);
    // ԥɤդ(\n)

void string_pomch(string_obj* str);
    // ԥɤդ(ԥɤΤޤ)

BOOL xclose(int fd);
    // closeȤˤΥեǥץ
    // gFReadBuffersΥХåեк
    // gFWriteBuffersΥХåեн񤭹Ǻ
    // gFileHandles˥եϥɥ뤬к

void update_ary_env(char* key);
    // ѿˤޤĤѿ(_size)ͤ˹

void clear_ary_env(char* key);
    // ѿˤޤĤѿä

void saphire_delete_same_name_var(char* name);
    // Ʊ̾ΥХϤѿƾä

int get_command_kind(char* arg0);
    // ޥ̾饳ޥIDõ

void null_fun();
    // gotoΥ٥򥳥ѥ顼̤δؿ

BOOL parse_is_redirect(char* p);
    // ݥ󥿡ʸ󤬥쥯Ȥɤ

BOOL parse_is_redirect2(char* p);
    // ݥ󥿤ʸ󤬥쥯Ȥɤ2

BOOL is_env_name_char(char c);
    // ѿǻȤʸɤ

BOOL is_glob_terminated_char(char* p);
    // ֤ζڤʸɤ

BOOL is_terminated_char(char* p);
    // 

void get_quoted_linefield(char* str, string_obj* out);

typedef struct _sJob {
    string_obj* mName;          // ֤̾
    pid_t mPGroup;              // ץ롼ID
    vector_obj* mPIDs;          // ҥץPIDΥꥹ
    struct termios* mTty;
} sJob;

sJob* sJob_new();
void sJob_delete(sJob* self);

enum eStatmentTerminated { kTNormal, kTOrOr, kTAndAnd };

typedef struct
{
    sStatments* joukensiki;
    sStatments* contents;
} sWhile;

sWhile* sWhile_new(MANAGED sStatments* joukensiki, MANAGED sStatments* contents);

void sWhile_delete(sWhile* self);
void sWhile_save(sWhile* self, int fd);
sWhile* sWhile_load(int fd);

typedef struct
{
    sStatments* contents;
} sSubshell;

sSubshell* sSubshell_new(MANAGED sStatments* contents);

void sSubshell_delete(sSubshell* self);
void sSubshell_save(sSubshell* self, int fd);
sSubshell* sSubshell_load(int fd);
void sSubshell_view(sSubshell* self);

typedef struct
{
    vector_obj* joukensiki_list;
    vector_obj* contents_list;
} sIf;

sIf* sIf_new(MANAGED vector_obj* joukensiki_list, MANAGED vector_obj* contents_list);

void sIf_delete(sIf* self);
void sIf_save(sIf* self, int fd);
sIf* sIf_load(int fd);

enum eCommandKind2 {
    kCKLocalVar, kCKUserCommand, kCKExternInnerCommand, kCKInnerCommand,
    kCKExternCommand
};

enum eRedirect { kRedirectInput, kRedirectOverwrite, kRedirectAppend
          , kRedirectErrAndOutput };

typedef struct {
    enum eRedirect mType;
    int mFd;
    string_obj* mFName;
} sRedirect; // ¹Ի˻Ȥ ֥ȥե¸

sRedirect* sRedirect_new(enum eRedirect type, int fd, char* fname);
void sRedirect_save(sRedirect* self, int fd);
sRedirect* sRedirect_load(int fd);
void sRedirect_delete(sRedirect* self);

typedef struct {
    enum eRedirect mType;
    int mFd;
} sRedirectTmp; // ѡǰŪ˻Ȥ

sRedirectTmp* sRedirectTmp_new(enum eRedirect type, int fd);
void sRedirectTmp_delete(sRedirectTmp* self);

typedef struct {
    string_obj* mName;

    string_obj* mIndex;
    string_obj* mIndex2;
    string_obj* mDelimiter;
} sVar;

sVar* sVar_new(MANAGED string_obj* name,  MANAGED string_obj* index, MANAGED string_obj* index2, MANAGED string_obj* delimiter);
void sVar_delete(sVar* self);

sVar* sVar_load(int fd);
void sVar_save(sVar* self, int fd);

typedef struct
{
    int mKind;   // 0: string 1: var 2: redirect 3: statments 4:var2 5: statments2
    void* mBody;
} sArgTmp; // ѡǰŪ˻Ȥ

sArgTmp* sArgTmp_new(int kind, void* body);
void sArgTmp_delete(sArgTmp* arg);

typedef struct
{
    int mKind; // 0:string 1: var 2: statments 3:var2 4: statments2
    void* mBody;
} sArg; // mArgs

sArg* sArg_new(int kind, void* body);
void sArg_delete(sArg* arg);

void sArg_save(sArg* self, int fd);
sArg* sArg_load(int fd);

typedef struct
{
    enum eCommandKind mKind;
    vector_obj* mArgs;
        // vector of sArg*
        // ޥɥ饤
    vector_obj* mRedirects;
        // vector of sRedirect* 
        // 쥯Υꥹ

    void* mExtra;

    vector_obj* mArgTmps;
        // vector of sArgTmp*
        // ѡѤ˰Ū˻Ȥѿ
} sCommand;

#ifdef MDEBUG 
sCommand* sCommand_new(const char* file, int line, const char* function);
#define COMMAND_NEW() sCommand_new(__FILE__, __LINE__, __FUNCTION__)
void sCommand_view(sCommand* self);
#else
sCommand* sCommand_new();
#define COMMAND_NEW() sCommand_new()
#endif

void sCommand_save(sCommand* self, int fd);
sCommand* sCommand_load(int fd);
void sCommand_delete(sCommand* self);

typedef struct
{
    vector_obj* mCommands;   // ޥ

    string_obj* mFName;      // ʸե̾
    int mLine;               // ʸ

    string_obj* mNotEvaled;  // run˥ѡʸ
                             // Ķѿȥޥޤʸ
    string_obj* mTitle;      // ȥ

    enum eStatmentTerminated mTerminated;   
            // ʣʸκǸ夬ǶڤƤ뤫

    BOOL mRCodeReverse;      // ꥿󥳡ɤȿž

    BOOL mBackground;        // Хå饦ɤǼ¹

    BOOL mGlobalPipeIn;   // ХѥפϤ뤫
    BOOL mGlobalPipeOut;  // ХѥפνϤ뤫
} sStatment;

#ifdef MDEBUG

sStatment* sStatment_new(const char* file, int line, const char* function);
#define STATMENT_NEW() sStatment_new(__FILE__, __LINE__, __FUNCTION__)
void sStatment_view(sStatment* self);

#else

sStatment* sStatment_new();
#define STATMENT_NEW() sStatment_new()

#endif

void sStatment_delete(sStatment* self);
void sStatment_save(sStatment* self, int fd);
sStatment* sStatment_load(int fd);

#ifdef MDEBUG
sStatments* sStatments_new(const char* file, int line, const char* function);

#define STATMENTS_NEW() sStatments_new(__FILE__, __LINE__, __FUNCTION__)
#else

sStatments* sStatments_new();

#define STATMENTS_NEW() sStatments_new()

#endif

void sStatments_view(sStatments* self);

void sStatments_save(sStatments* self, int fd);
sStatments* sStatments_load(int fd);
void sStatments_delete(sStatments* self);

typedef struct {
    string_obj* name;       // ޥ̾
    string_obj* arg_name;
    sStatments* statments; // ̿ɽ
} sFunction;

sFunction* sFunction_new(char* name, MANAGED sStatments* statments, char* arg_name);
  
void sFunction_delete(sFunction* self);
void sFunction_save(sFunction* self, int fd);
sFunction* sFunction_load(int fd);
void sFunction_view(sFunction* self);

typedef struct
{
    int mPid;
    int mStatus;
    int mPGroup;
} sOtherCProg;

extern vector_obj* gOtherCProgs;

sOtherCProg* sOtherCprog_new(int pid, int status, int pgroup);
void sOtherCProg_delete(sOtherCProg* self);

typedef struct {
    pid_t mPid;
    string_obj* mTitle;
    BOOL mWriterProcess;
} sChildProgram;

sChildProgram* sChildProgram_new(pid_t pid, char* title, BOOL writer_process);
void sChildProgram_delete(sChildProgram* self);

typedef struct {
    string_obj* mName;
    fInnerCommand mFun;
} sInnerCommand;

sInnerCommand* sInnerCommand_new(char* name, fInnerCommand fun);
void sInnerCommand_delete(sInnerCommand* self);

void sigchld_block(int block);

////////////////////////////////////////////////////////////
/// from saphire_commands.c
////////////////////////////////////////////////////////////
BOOL statment_tree_internal_commands(sCommand* command, int* rcode, vector_obj* argv, sWFd* nextout, sRFd* nextin, int nexterr, int j, vector_obj* cprogs, char* title, BOOL input);
BOOL vcat_expand_env(string_obj* pipeout, char* env, vector_obj* numbers);

////////////////////////////////////////////////////////////
/// from saphire_parser.c
////////////////////////////////////////////////////////////
BOOL expand_env(char* p, string_obj* cmdline2);
int parse(char* cmdline, char* fname, sStatments* statments);
BOOL read_expand_brace_expansion(char* p, string_obj* cmdline2, char* separator);
BOOL parse_number(char* p, vector_obj* numbers);

typedef struct {
    int number;
    int number2;
    int number_num;
} sIndex;

sIndex* sIndex_new(int num, int num2, int num_num);
void sIndex_delete(sIndex* self);

BOOL read_until_next_cbrace_end(char** p, string_obj* not_evaled, int* read_end_of_statment);

////////////////////////////////////////////////////////////
// from saphire_vm.c
////////////////////////////////////////////////////////////
BOOL forground_job(int num);
void background_job(int num);
void tmpfile_all_sweep();

#endif
